#!/usr/bin/python
# -*- coding: utf8 -*-
#
#  vim: expandtab:tw=68:ts=4:sw=4:
#
#  (c) Copyright 2008 Samsung Telecommunications America, Inc.
#                   All rights reserved
#
#              Wireless Terminals Lab R & D
# 
# 
#  File: install_gtk.py
#  Author: Mohan Reddy Duggi (mreddy@sta.samsung.com)
#  Creation date: Wed May 21 13:37:07 2008
#  Rev: $Id$
#
#  Rev: 1.0, 17 october 2008 by Philippe Corbes (philippe.corbes@logibag.com)
#        Add unzip file management
#        Add download_and_unpack_patch
#
#  Rev: 1.1, 06 November 2008 by Philippe Corbes (philippe.corbes@logibag.com)
#        Add --list, --strip, --disable-patch --log and --html options
#        Add export environement variable from packet parameters before make
#        Add up to 4 substitutions in export environement vars
#

import sys;
import os;
import getopt;
import string;
import urllib;
import time;
import locale;

from posixpath import exists;

from config import Pkg_info;
from config import Env_kvp;


#----------------------------------------------------------------------------
def usage(prog_name) :
#----------------------------------------------------------------------------
    usage_str = """
$prog_name: A script to download, patch, build and install gtk and its dependencies automatically.

Usage: $prog_name [Options]

Options:
  --help        Show this help message and quit.
  --install-prefix=p    Path where the gtk libs should be installed.
                        Default value: /usr/work/dfb_arm
  --tools-prefix=p  Prefix of commands like CC. May include a complete path
  --src-prefix=p    Path where the pre-downloaded gtk source directories exist.
                    Default value: ./
  --target-root=p
  --disable-patch   Disable path. Very usefull to apply only once
  --patch-prefix=p  Path where the patches exist
  --list            List all avalaible pakages
  --only=pkg        Build only a single package
  --start-with-pkg=name  Start building with the given package
  --clean           Execute "make distclean" before building the package
  --fresh-build     Remove the un-packed directories and start fresh
  --strip           Execute a "make install-strip" instead of a "make install"
  --html            Output an html file to discribe all packages and commands will
                    be executed to build packages.
  --log=file        log "make" and "make install" output in the specified file


Example:

0) Check Download, patch, build and install packages before installation

   $prog_name --html --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk --patch-prefix=/root/patches > install.html

1) Download, patch, build and install packages (Generally for the first time)

   $prog_name --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk --patch-prefix=/root/patches

2) Build & install packages without patching

   $prog_name --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk

3) Start build from specified package

   $prog_name --install-prefix=/usr/work/dfb --src-prefix=/root/packages --clean

4) Remove already unpacked packages and do a fresh build

   $prog_name --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk --patch-prefix=/root/patches --fresh-build

5) Clean build and install packages

   $prog_name --install-prefix=/usr/work/dfb --src-prefix=/usr/work/gtk --clean

""";

    prog_name = prog_name[prog_name.rfind ("/") + 1:];
    str_list = usage_str.split ("$prog_name");

    print prog_name.join (str_list);


#----------------------------------------------------------------------------
def exec_shell_cmd (cmd) :
#----------------------------------------------------------------------------
    print "Executing: " + cmd + "\n";

    if os.system (cmd) != 0 :
        print "Command " + cmd + " failed\n";
        sys.exit(3);


#----------------------------------------------------------------------------
def make_abs_prefix (prefix) :
#----------------------------------------------------------------------------
    if prefix[0] != "/" :
        prefix = os.getcwd () + "/" + prefix;

    return prefix;


#----------------------------------------------------------------------------
def parse_args (prog_name, args, conf) :
#----------------------------------------------------------------------------
    try :
        opts, args = getopt.getopt (args, "", ["help", "install-prefix=", "src-prefix=", "target-root=", "tools-prefix=", "patch-prefix=", "start-with-pkg=", "fresh-build", "clean", "only=", "strip", "list", "disable-patch", "log=", "html"]);
    except getopt.GetoptError:
        print "Invalid arguments passed. Try " +  prog_name + " --help";
        usage (prog_name);
        sys.exit(2);

    conf["install-prefix"] = "/usr/work/dfb_arm";
    conf["target-root"] = "";
    conf["tools-prefix"] = "";
    conf["src-prefix"] = os.getcwd ();
    conf["patch-prefix"] = "";
    conf["start-with-pkg"] = "";
    conf["fresh-build"] = 0;
    conf["clean"] = 0;
    conf["only"] = "";
    conf["strip"] = 0;
    conf["list"] = 0;
    conf["disable-patch"] = 0;
    conf["log"] = "";
    conf["html"] = 0;

    for opt, arg in opts :
        if opt == "--help" :
            usage (prog_name);
            sys.exit(2);
        elif opt == "--install-prefix" :
            conf["install-prefix"] = make_abs_prefix (arg);
        elif opt == "--target-root" :
            conf["target-root"] = make_abs_prefix (arg);
        elif opt == "--tools-prefix" :
            conf["tools-prefix"] = make_abs_prefix (arg);
        elif opt == "--src-prefix" :
            conf["src-prefix"] = make_abs_prefix (arg);
        elif opt == "--patch-prefix" :
            conf["patch-prefix"] = make_abs_prefix (arg);
        elif opt == "--start-with-pkg" :
            conf["start-with-pkg"] = arg;
        elif opt == "--only" :
            conf["only"] = arg;
        elif opt == "--disable-patch" :
            conf["disable-patch"] = 1;
        elif opt == "--fresh-build" :
            conf["fresh-build"] = 1;
        elif opt == "--clean" :
            conf["clean"] = 1;
        elif opt == "--html" :
            conf["html"] = 1;
        elif opt == "--list" :
            conf["list"] = 1;
        elif opt == "--log" :
            conf["log"] = arg;
        elif opt == "--strip" :
            conf["strip"] = 1;

    if conf["install-prefix"] == "":
        print "\n--install-prefix option is necessary\n";
        sys.exit(2);

    if conf["start-with-pkg"] != "" and conf["only"] != "" :
        print "\n--start-with-pkg and --only options shouldn't be given same time\n";
        sys.exit(2);


#----------------------------------------------------------------------------
def env_substitute (conf, aString) :
#----------------------------------------------------------------------------
    # Substitute "install_prefix", Up to 4 substitutions !!!
    changeTo = conf["install-prefix"]
    vals = aString.split ("${install_prefix}");
    lg = len (vals);
    if lg > 5 :
        print "Impossible to substitute $install_prefix more than 4 time in " + aSting;
        sys.exit(2);
    elif lg == 1 :
        vals = vals[0];
    elif lg == 2 :
        vals = vals[0] + changeTo + vals[1];
    elif lg == 3 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2];
    elif lg == 4 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2] + changeTo + vals[3];
    elif lg == 5 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2] + changeTo + vals[3] + changeTo + vals[4];

    # Substitute "tools_prefix", Up to 4 substitutions !!!
    aString = vals
    changeTo = conf["tools-prefix"]
    vals = aString.split ("${tools_prefix}");
    lg = len (vals);
    if lg > 5 :
        print "Impossible to substitute $tools_prefix more than 4 time in " + aSting;
        sys.exit(2);
    elif lg == 1 :
        vals = vals[0];
    elif lg == 2 :
        vals = vals[0] + changeTo + vals[1];
    elif lg == 3 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2];
    elif lg == 4 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2] + changeTo + vals[3];
    elif lg == 5 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2] + changeTo + vals[3] + changeTo + vals[4];

    # Substitute "tools_prefix", Up to 4 substitutions !!!
    aString = vals
    changeTo = conf["target-root"]
    vals = aString.split ("${target_root}");
    lg = len (vals);
    if lg > 5 :
        print "Impossible to substitute $target_root more than 4 time in " + aSting;
        sys.exit(2);
    elif lg == 1 :
        vals = vals[0];
    elif lg == 2 :
        vals = vals[0] + changeTo + vals[1];
    elif lg == 3 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2];
    elif lg == 4 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2] + changeTo + vals[3];
    elif lg == 5 :
        vals = vals[0] + changeTo + vals[1] + changeTo + vals[2] + changeTo + vals[3] + changeTo + vals[4];

    return vals


#----------------------------------------------------------------------------
def dfb_env_setup (conf) :
#----------------------------------------------------------------------------
    old_dir = os.getcwd();
    prefix = conf["install-prefix"];

    if conf["html"] :
        print "<ul>";
        print "<li>mkdir " + prefix + "</li>";
        print "<li>cd " + prefix + "</li>";
        print "<li>mkdir -p man/man1 bin lib etc include info sbin share src</li>";
        print "<li>cd " + old_dir + "</li>";
    else :
        if os.path.exists (prefix) == 0 :
            os.mkdir (prefix);
            os.chdir (prefix);
            exec_shell_cmd ('mkdir -p man/man1 bin lib etc include info sbin share src');
            os.chdir (old_dir);

    for key in Env_kvp.keys () :
        vals = env_substitute (conf, Env_kvp[key]);

        if os.environ.has_key (key) :
            if key.find ("PATH") < 0 and key.find ("LD_LIBRARY_PATH") < 0 :
                os.environ[key] =  vals + " " + os.environ[key];
            else :
                os.environ[key] =  vals + ":" + os.environ[key];
        else :
            os.environ[key] = vals;

        if conf["html"] :
            print "<li>export " + key + "=" + os.environ[key] + "</li>";
        else :
            print "export " + key + "=" + os.environ[key] + "\n";

    if conf["html"] :
        print "</ul>";


#----------------------------------------------------------------------------
def export_pkg (pkg, conf) :
#----------------------------------------------------------------------------
    for key in pkg["export"].keys () :
        vals = env_substitute (conf, pkg["export"][key]);

        if os.environ.has_key (key) :
            if key.find ("PATH") < 0 and key.find ("LD_LIBRARY_PATH") < 0 :
                os.environ[key] =  vals + " " + os.environ[key];
            else :
                os.environ[key] =  vals + ":" + os.environ[key];
        else :
            os.environ[key] = vals;

        if conf["html"] :
            print "<li>export " + key + "=" + os.environ[key] + "</li>";
        else :
            print "export " + key + "=" + os.environ[key] + "\n";


#----------------------------------------------------------------------------
def download_and_unpack_pkg (pkg, conf) :
#----------------------------------------------------------------------------
    #If the package directory already exists, don't download
    if exists (conf["src-prefix"] + "/" + pkg["name"]) :
        return;

    url = pkg["download"];
    file = url[url.rfind ("/") + 1:];

    if not conf["html"] :
        #If the package doesn't exist, download it.
        if exists (file) == 0 :
            print "Downloading " + url + " to " + file;
            urllib.urlretrieve (url, file);

    type = file[file.rfind (".") + 1:];

    if type == "bz2" :
        cmd = "tar -xjvf " + file;
    elif type == "gz" :
        cmd = "tar -xzvf " + file;
    elif type == "zip" :
        cmd = "unzip " + file + " -d " + conf["src-prefix"];

    if conf["html"] :
        print "<li>" + cmd + "</li>";
    else :
        exec_shell_cmd (cmd);


#----------------------------------------------------------------------------
def download_and_unpack_patch (conf, url, file, patch_file) :
#----------------------------------------------------------------------------
    #If the patch file already exists, don't download
    if exists (patch_file) :
        return;

    if not conf["html"] :
        #If the patch doesn't exist, download it.
        if exists (file) == 0 :
            print "Downloading " + url + " to " + file;
            urllib.urlretrieve (url, file);

        if file.endswith(".zip") :
            cmd = "unzip " + file + " -d " + conf["patch-prefix"];

        exec_shell_cmd (cmd);


#----------------------------------------------------------------------------
def patch_pkg (pkg, conf) :
#----------------------------------------------------------------------------
    for patch in pkg["patches"] :
        url = patch["url"];
        file = conf["patch-prefix"] + "/" + url[url.rfind ("/") + 1:];
        patch_file =  conf["patch-prefix"] + "/" + patch["patch"];

        download_and_unpack_patch (conf, url, file, patch_file);

        patch_cmd = "patch -p1 < " + patch_file;

        if conf["html"] :
            print "<li>" + patch_cmd + "</li>";
        else :
            exec_shell_cmd (patch_cmd);


#----------------------------------------------------------------------------
def build_pkg (pkg, conf) :
#----------------------------------------------------------------------------
    if conf["clean"] :
        try :
            clean_cmd ="make distclean";
            if conf["html"] :
                print "<li>" + clean_cmd + "</li>";
            else :
                exec_shell_cmd (clean_cmd);
        except :
            print "Ignoring make distclean failure..\n";

    conf_opts = "";
    if pkg.has_key ("config-opts") :
        conf_opts = pkg["config-opts"];

    config_cmd = "./configure " + "--prefix=" + conf["install-prefix"] + " " + conf_opts;

    if conf["html"] :
        print "<li>" + config_cmd + "</li>";
    else :
        print config_cmd + "\n";
        exec_shell_cmd (config_cmd);


    if conf["log"] != "":
        if pkg.has_key ("make") :
            make_cmd = pkg["make"] + " >" + conf["log"];
        else :
            make_cmd = "make >" + conf["log"];
    else :
        if pkg.has_key ("make") :
            make_cmd = pkg["make"];
        else :
            make_cmd = "make";

    if conf["html"] :
        print "<li>" + make_cmd + "</li>";
    else :
        print make_cmd + "\n";
        exec_shell_cmd (make_cmd);


#----------------------------------------------------------------------------
def install_pkg (pkg, conf) :
#----------------------------------------------------------------------------
    if conf["log"] != "":
        if conf["strip"] :
            install_cmd1 = "make install-strip >>" + conf["log"];
            install_cmd2 = "make install >>" + conf["log"];
        else :
            install_cmd1 = "make install >>" + conf["log"];
    else :
        if conf["strip"] :
            install_cmd1 = "make install-strip";
            install_cmd2 = "make install";
        else :
            install_cmd1 = "make install";

    if conf["html"] :
        print "<li>" + install_cmd1 + "</li>";
    else :
        if conf["strip"] :
            try :
                print install_cmd1 + "\n";
                exec_shell_cmd (install_cmd1);
            except :
                print install_cmd2 + "\n";
                exec_shell_cmd (install_cmd2);
        else :
            print install_cmd1 + "\n";
            exec_shell_cmd (install_cmd1);


#----------------------------------------------------------------------------
# main
#----------------------------------------------------------------------------
if __name__ == "__main__" :
    Conf = {};
    start_flag = 1;
    parse_args (sys.argv[0], sys.argv[1:], Conf);

    if Conf["list"] :
        print "# " + sys.argv[0] + " - Package list:";
        for pkg in Pkg_info :
            print pkg["name"];

    else :

        if Conf["html"] :
            encoding = locale.getdefaultlocale();
            print "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=" + encoding[1] + "\"><title>GTK-DFB Package Installation from sources</title><meta name=\"generator\" content=\"" + sys.argv[0] + "\"><meta name=\"author\" content=\"philippe Corbes\"></head><body>";
            print "<h1> GTK-DFB Package Installation from sources</h1>";
            print "<b>Building command:</b> "
            for arg in sys.argv :
                print "&nbsp;&nbsp;&nbsp;&nbsp;" + arg + "<br>";
            print "<br><b>Notice:</b> The execution of this command is detailed in the lines below. I suggest you to check this lines before installation.";
            print "<hr>";
            print "<h1>Preparing installation</h1>";

        dfb_env_setup (Conf);

        if Conf["html"] :
            print "<hr>";
            print "<h1>Installing packages</h1>";

        if Conf["start-with-pkg"] != "" :
            start_flag = 0;

        package_number = 1;
        for pkg in Pkg_info :

            if start_flag == 0 and pkg["name"] == Conf["start-with-pkg"] :
                start_flag = 1;

            if start_flag == 1 :

                if Conf["only"] != "" and pkg["name"] != Conf["only"] :
                    continue;

                if Conf["html"] :
                    print "<h2>" + str(package_number) +") " + pkg["name"] + "</h1>";
                    print "<ul><li> <b>Package Name: </b> " + pkg["name"] + "</li>";
                    print "<li><b>Download URL: </b> <a href=\"" + pkg["download"] + "\" title=\"" + pkg["download"] + "\">" + pkg["download"] + "</a></li>";
                    if pkg.has_key ("patches") :
                        print "<li><b>Patches: </b> Download following patches.<ul>";
                        for patch in pkg["patches"] :
                            url = patch["url"];
                            file = url[url.rfind ("/") + 1:];
                            print " <li> <a href=\"" + url + "\" title=\"" + url + "\">" + file + "</a></li>";
                        print "</ul></li>";
                    print "<li><b>Installation: </b><ul>";
                else :
                    print "\n###\n###    Package: " + pkg["name"] + "\n###\n";

                    os.chdir (Conf["src-prefix"]);

                package_number += 1;

                if Conf["fresh-build"] == 1 :
                    rm_cmd ="rm -rf " + pkg["name"];
                    if Conf["html"] :
                        print "<li>" + rm_cmd + "</li>";
                    else :
                        exec_shell_cmd (rm_cmd);

                download_and_unpack_pkg (pkg, Conf);

                if Conf["html"] :
                    print "<li>cd " + pkg["name"] + "</li>";
                else :
                    os.chdir (Conf["src-prefix"] + "/"+ pkg["name"]);

                if not Conf["disable-patch"] and Conf["patch-prefix"] != "" and pkg.has_key ("patches") :
                    patch_pkg (pkg, Conf);

                if pkg.has_key ("export") :
                    export_pkg (pkg, Conf);

                build_pkg (pkg, Conf);

                install_pkg (pkg, Conf);

                if Conf["html"] :
                    print "</ul>";
                    print "</li></ul>";

        if Conf["html"] :
            print "<hr>"
            print time.strftime("<i>%d %b %Y - %H:%M:%S", time.localtime()) + " - This document has been automaticaly generated by \"" + sys.argv[0] + " --html ....\"</i>";
            print "</body></html>";

#----------------------------------------------------------------------------
# EOF

